@@ -27,28 +27,34 @@
 
 DataManager::DataManager()
 {
-    connect(
-        &Fetcher::instance(),
-        &Fetcher::feedDetailsUpdated,
-        this,
-        [this](const QString &url, const QString &name, const QString &image, const QString &link, const QString &description, const QDateTime &lastUpdated) {
-            qCDebug(kastsDataManager) << "Start updating feed details for" << url;
-            Feed *feed = getFeed(url);
-            if (feed != nullptr) {
-                feed->setName(name);
-                feed->setImage(image);
-                feed->setLink(link);
-                feed->setDescription(description);
-                feed->setLastUpdated(lastUpdated);
-                qCDebug(kastsDataManager) << "Retrieving authors";
-                feed->updateAuthors();
-                // For feeds that have just been added, this is probably the point
-                // where the Feed object gets created; let's set refreshing to
-                // true in order to show user feedback that the feed is still
-                // being fetched
-                feed->setRefreshing(true);
-            }
-        });
+    connect(&Fetcher::instance(),
+            &Fetcher::feedDetailsUpdated,
+            this,
+            [this](const QString &url,
+                   const QString &name,
+                   const QString &image,
+                   const QString &link,
+                   const QString &description,
+                   const QDateTime &lastUpdated,
+                   const QString &dirname) {
+                qCDebug(kastsDataManager) << "Start updating feed details for" << url;
+                Feed *feed = getFeed(url);
+                if (feed != nullptr) {
+                    feed->setName(name);
+                    feed->setImage(image);
+                    feed->setLink(link);
+                    feed->setDescription(description);
+                    feed->setLastUpdated(lastUpdated);
+                    feed->setDirname(dirname);
+                    qCDebug(kastsDataManager) << "Retrieving authors";
+                    feed->updateAuthors();
+                    // For feeds that have just been added, this is probably the point
+                    // where the Feed object gets created; let's set refreshing to
+                    // true in order to show user feedback that the feed is still
+                    // being fetched
+                    feed->setRefreshing(true);
+                }
+            });
     connect(&Fetcher::instance(), &Fetcher::entryAdded, this, [this](const QString &feedurl, const QString &id) {
         Q_UNUSED(feedurl)
         // Only add the new entry to m_entries
@@ -190,17 +196,6 @@ int DataManager::entryCount(const Feed *feed) const
     return m_entrymap[feed->url()].count();
 }
 
-int DataManager::newEntryCount(const Feed *feed) const
-{
-    QSqlQuery query;
-    query.prepare(QStringLiteral("SELECT COUNT (id) FROM Entries where feed=:feed AND new=1;"));
-    query.bindValue(QStringLiteral(":feed"), feed->url());
-    Database::instance().execute(query);
-    if (!query.next())
-        return -1;
-    return query.value(0).toInt();
-}
-
 void DataManager::removeFeed(Feed *feed)
 {
     QList<Feed *> feeds;
@@ -278,6 +273,11 @@ void DataManager::removeFeeds(const QList<Feed *> &feeds)
             m_entrymap.remove(feedurl); // remove all the entry mappings belonging to the feed
 
             qCDebug(kastsDataManager) << "Remove feed image" << feed->image() << "for feed" << feedurl;
+            qCDebug(kastsDataManager) << "Remove feed enclosure download directory" << feed->dirname() << "for feed" << feedurl;
+            QDir enclosureDir = QDir(StorageManager::instance().enclosureDirPath() + feed->dirname());
+            if (!feed->dirname().isEmpty() && enclosureDir.exists()) {
+                enclosureDir.removeRecursively();
+            }
             if (!feed->image().isEmpty())
                 StorageManager::instance().removeImage(feed->image());
             m_feeds.remove(m_feedmap[index]); // remove from m_feeds
@@ -351,7 +351,8 @@ void DataManager::addFeeds(const QStringList &urls, const bool fetch)
     // TODO: Add more checks like checking if URLs exist; however this will mean async...
     QStringList newUrls;
     for (const QString &url : urls) {
-        if (!url.trimmed().isEmpty()) {
+        if (!url.trimmed().isEmpty() && !feedExists(url)) {
+            qCDebug(kastsDataManager) << "Feed already exists or URL is empty" << url.trimmed();
             newUrls << url.trimmed();
         }
     }
@@ -364,18 +365,13 @@ void DataManager::addFeeds(const QStringList &urls, const bool fetch)
     // authors and enclosures) will be updated by calling Fetcher::fetch() which
     // will trigger a full update of the feed and all related items.
     for (const QString &url : newUrls) {
-        qCDebug(kastsDataManager) << "Adding feed";
-        if (feedExists(url)) {
-            qCDebug(kastsDataManager) << "Feed already exists";
-            continue;
-        }
-        qCDebug(kastsDataManager) << "Feed does not yet exist";
+        qCDebug(kastsDataManager) << "Adding new feed:" << url;
 
         QUrl urlFromInput = QUrl::fromUserInput(url);
         QSqlQuery query;
         query.prepare(
             QStringLiteral("INSERT INTO Feeds VALUES (:name, :url, :image, :link, :description, :deleteAfterCount, :deleteAfterType, :subscribed, "
-                           ":lastUpdated, :new, :notify);"));
+                           ":lastUpdated, :new, :notify, :dirname);"));
         query.bindValue(QStringLiteral(":name"), urlFromInput.toString());
         query.bindValue(QStringLiteral(":url"), urlFromInput.toString());
         query.bindValue(QStringLiteral(":image"), QLatin1String(""));
@@ -387,6 +383,7 @@ void DataManager::addFeeds(const QStringList &urls, const bool fetch)
         query.bindValue(QStringLiteral(":lastUpdated"), 0);
         query.bindValue(QStringLiteral(":new"), true);
         query.bindValue(QStringLiteral(":notify"), false);
+        query.bindValue(QStringLiteral(":dirname"), QLatin1String(""));
         Database::instance().execute(query);
 
         m_feeds[urlFromInput.toString()] = nullptr;
@@ -498,6 +495,48 @@ void DataManager::removeFromQueue(const QString &id)
     Q_EMIT queueEntryRemoved(index, id);
 }
 
+void DataManager::sortQueue(AbstractEpisodeProxyModel::SortType sortType)
+{
+    QString columnName;
+    QString order;
+
+    switch (sortType) {
+    case AbstractEpisodeProxyModel::SortType::DateAscending:
+        order = QStringLiteral("ASC");
+        columnName = QStringLiteral("updated");
+        break;
+    case AbstractEpisodeProxyModel::SortType::DateDescending:
+        order = QStringLiteral("DESC");
+        columnName = QStringLiteral("updated");
+        break;
+    }
+
+    QStringList newQueuemap;
+
+    QSqlQuery query;
+    query.prepare(QStringLiteral("SELECT * FROM Queue INNER JOIN Entries ON Queue.id = Entries.id ORDER BY %1 %2;").arg(columnName, order));
+    Database::instance().execute(query);
+
+    while (query.next()) {
+        qCDebug(kastsDataManager) << "new queue order:" << query.value(QStringLiteral("id")).toString();
+        newQueuemap += query.value(QStringLiteral("id")).toString();
+    }
+
+    Database::instance().transaction();
+    for (int i = 0; i < m_queuemap.length(); i++) {
+        query.prepare(QStringLiteral("UPDATE Queue SET listnr=:listnr WHERE id=:id;"));
+        query.bindValue(QStringLiteral(":id"), newQueuemap[i]);
+        query.bindValue(QStringLiteral(":listnr"), i);
+        Database::instance().execute(query);
+    }
+    Database::instance().commit();
+
+    m_queuemap.clear();
+    m_queuemap = newQueuemap;
+
+    Q_EMIT queueSorted();
+}
+
 QString DataManager::lastPlayingEntry()
 {
     QSqlQuery query;
@@ -526,8 +565,11 @@ void DataManager::setLastPlayingEntry(const QString &id)
 void DataManager::deletePlayedEnclosures()
 {
     QSqlQuery query;
-    query.prepare(QStringLiteral("SELECT * FROM Enclosures INNER JOIN Entries ON Enclosures.id = Entries.id WHERE downloaded=:downloaded AND read=:read;"));
+    query.prepare(
+        QStringLiteral("SELECT * FROM Enclosures INNER JOIN Entries ON Enclosures.id = Entries.id WHERE"
+                       "(downloaded=:downloaded OR downloaded=:partiallydownloaded) AND (read=:read);"));
     query.bindValue(QStringLiteral(":downloaded"), Enclosure::statusToDb(Enclosure::Downloaded));
+    query.bindValue(QStringLiteral(":partiallydownloaded"), Enclosure::statusToDb(Enclosure::PartiallyDownloaded));
     query.bindValue(QStringLiteral(":read"), true);
     Database::instance().execute(query);
     while (query.next()) {
@@ -686,6 +728,22 @@ void DataManager::bulkMarkNew(bool state, QStringList list)
     Q_EMIT bulkNewStatusActionFinished();
 }
 
+void DataManager::bulkMarkFavoriteByIndex(bool state, QModelIndexList list)
+{
+    bulkMarkFavorite(state, getIdsFromModelIndexList(list));
+}
+
+void DataManager::bulkMarkFavorite(bool state, QStringList list)
+{
+    Database::instance().transaction();
+    for (QString id : list) {
+        getEntry(id)->setFavoriteInternal(state);
+    }
+    Database::instance().commit();
+
+    Q_EMIT bulkFavoriteStatusActionFinished();
+}
+
 void DataManager::bulkQueueStatusByIndex(bool state, QModelIndexList list)
 {
     bulkQueueStatus(state, getIdsFromModelIndexList(list));
