@@ -15,6 +15,7 @@
 #include <QFile>
 #include <QFileInfo>
 #include <QNetworkAccessManager>
+#include <QNetworkProxyFactory>
 #include <QNetworkReply>
 #include <QTime>
 #include <QTimer>
@@ -24,12 +25,11 @@
 #include "fetchfeedsjob.h"
 #include "kasts-version.h"
 #include "models/errorlogmodel.h"
+#include "networkconnectionmanager.h"
 #include "settingsmanager.h"
 #include "storagemanager.h"
 #include "sync/sync.h"
 
-#include <solidextras/networkstatus.h>
-
 Fetcher::Fetcher()
 {
     connect(this, &Fetcher::error, &ErrorLogModel::instance(), &ErrorLogModel::monitorErrorMessages);
@@ -42,6 +42,8 @@ Fetcher::Fetcher()
     manager->setRedirectPolicy(QNetworkRequest::NoLessSafeRedirectPolicy);
     manager->setStrictTransportSecurityEnabled(true);
     manager->enableStrictTransportSecurityStore(true);
+
+    QNetworkProxyFactory::setUseSystemConfiguration(true);
 }
 
 void Fetcher::fetch(const QString &url)
@@ -127,9 +129,7 @@ QString Fetcher::image(const QString &url)
 
     // if image has not yet been cached, then check for network connectivity if
     // possible; and download the image
-    SolidExtras::NetworkStatus networkStatus;
-    if (networkStatus.connectivity() == SolidExtras::NetworkStatus::No
-        || (networkStatus.metered() == SolidExtras::NetworkStatus::Yes && !SettingsManager::self()->allowMeteredImageDownloads())) {
+    if (!NetworkConnectionManager::instance().imageDownloadsAllowed()) {
         return QLatin1String("no-image");
     }
 
