@@ -33,6 +33,7 @@
 #include "fetcher.h"
 #include "fetchfeedsjob.h"
 #include "models/errorlogmodel.h"
+#include "networkconnectionmanager.h"
 #include "settingsmanager.h"
 #include "storagemanager.h"
 #include "sync/gpodder/devicerequest.h"
@@ -48,8 +49,6 @@
 #include "sync/syncjob.h"
 #include "sync/syncutils.h"
 
-#include <solidextras/networkstatus.h>
-
 using namespace SyncUtils;
 
 Sync::Sync()
@@ -101,9 +100,7 @@ void Sync::retrieveCredentialsFromConfig()
                 // Now that we have all credentials we can do the initial sync if
                 // it's enabled in the config.  If it's not enabled, then we handle
                 // the automatic refresh through main.qml
-                SolidExtras::NetworkStatus networkStatus;
-                if (networkStatus.connectivity() != SolidExtras::NetworkStatus::No
-                    && (networkStatus.metered() != SolidExtras::NetworkStatus::Yes || SettingsManager::self()->allowMeteredFeedUpdates())) {
+                if (NetworkConnectionManager::instance().feedUpdatesAllowed()) {
                     if (SettingsManager::self()->refreshOnStartup() && SettingsManager::self()->syncWhenUpdatingFeeds()) {
                         doRegularSync(true);
                     }
@@ -483,8 +480,8 @@ void Sync::savePasswordToFile(const QString &username, const QString &password)
                      passwordFile.fileName(),
                      QStringLiteral(""),
                      0,
-                     i18n("I/O Denied: Cannot save password."),
-                     i18n("I/O Denied: Cannot save password."));
+                     i18n("I/O denied: Cannot save password."),
+                     i18n("I/O denied: Cannot save password."));
         Q_EMIT passwordSaveFinished(false);
     } else {
         passwordFile.write(password.toUtf8());
@@ -564,8 +561,8 @@ QString Sync::retrievePasswordFromFile(const QString &username)
                      passwordFile.fileName(),
                      QStringLiteral(""),
                      0,
-                     i18n("I/O Denied: Cannot access password file."),
-                     i18n("I/O Denied: Cannot access password file."));
+                     i18n("I/O denied: Cannot access password file."),
+                     i18n("I/O denied: Cannot access password file."));
 
         return QStringLiteral("");
     }
@@ -814,9 +811,7 @@ void Sync::doQuickSync()
 
     // since this method is supposed to be called automatically, we cannot check
     // the network state from the UI, so we have to do it here
-    SolidExtras::NetworkStatus networkStatus;
-    if (networkStatus.connectivity() == SolidExtras::NetworkStatus::No
-        || (networkStatus.metered() == SolidExtras::NetworkStatus::Yes && !SettingsManager::self()->allowMeteredFeedUpdates())) {
+    if (!NetworkConnectionManager::instance().feedUpdatesAllowed()) {
         qCDebug(kastsSync) << "Not uploading episode actions on metered connection due to settings";
         return;
     }
@@ -867,9 +862,9 @@ void Sync::applyEpisodeActionsLocally(const QHash<QString, QHash<QString, Episod
                 Entry *entry = DataManager::instance().getEntry(action.id);
                 if (entry && entry->hasEnclosure()) {
                     qCDebug(kastsSync) << action.position << action.total << static_cast<qint64>(action.position) << entry->enclosure()->duration()
-                                       << AudioManager::instance().SKIP_TRACK_END / 1000;
-                    if ((action.position >= action.total - AudioManager::instance().SKIP_TRACK_END / 1000
-                         || static_cast<qint64>(action.position) >= entry->enclosure()->duration() - AudioManager::instance().SKIP_TRACK_END / 1000)
+                                       << SettingsManager::self()->markAsPlayedBeforeEnd();
+                    if ((action.position >= action.total - SettingsManager::self()->markAsPlayedBeforeEnd()
+                         || static_cast<qint64>(action.position) >= entry->enclosure()->duration() - SettingsManager::self()->markAsPlayedBeforeEnd())
                         && action.total > 0) {
                         // Episode has been played
                         qCDebug(kastsSync) << "mark as played:" << entry->title();
